import os
import simbricks.orchestration.simulators as sim
import simbricks.orchestration.nodeconfig as node


# System configuration for the simulated machine. Configures commands to run
# as the system boots before the application runs. Specifically this loads the
# vfio-pci kernel driver and registers our PCI vendor and device id with the
# vfio-pci driver.
class SwishNode(node.NodeConfig):
    def __init__(self):
        super().__init__()
        self.drivers = []
        self.memory = 2048
        self.kcmd_append = ' memmap=512M!1G'

    def prepare_pre_cp(self):
        l = [
            'mount -t proc proc /proc',
            'mount -t sysfs sysfs /sys',
            'echo 1 > /sys/module/vfio/parameters/enable_unsafe_noiommu_mode',
            'echo f001 1dea >/sys/bus/pci/drivers/vfio-pci/new_id']
        return super().prepare_pre_cp() + l


# Actual application to run. Includes the command to run, but also makes sure
# the compiled binary gets copied into the disk image of the simulated machine.
class VKernelLaunch(node.AppConfig):
    def __init__(self, kname = None):
        super().__init__()
        self.kernel = kname

    def run_cmds(self, node):
        return [f'LD_LIBRARY_PATH=/tmp/guest /tmp/guest/vortex-launch {self.kernel}']

    def config_files(self, env = None):
        # copy binary into host image during prep
        print(env)
        m = {'vortex-launch': open('hw/vortex_env/vortex-swish/build/tests/regression/'+self.kernel+'/'+self.kernel, 'rb'), 
             'kernel.vxbin' : open('hw/vortex_env/vortex-swish/build/tests/regression/'+self.kernel+'/kernel.vxbin', 'rb'),
             'libvortex.so' : open('hw/vortex_env/vortex-swish/build/runtime/simbricks/libvortex.so', 'rb')}
        return {**m, **super().config_files(env)}


# Simulator component for vortex accelerator model
class VortexSim(sim.PCIDevSim):
    sync = True

    def __init__(self):
        super().__init__()
        self.clock_period = 10000

    def run_cmd(self, env):
        cmd = '%s%s %d %s %s' % \
            (os.getcwd(), f'/hw/vortex_env/vortex-swish/build/sim/simbricks/vx-sbsim', self.clock_period,
             env.dev_pci_path(self), env.dev_shm_path(self))
        return cmd
